XK_hebrew_doublelowline = 0xcdf
XK_hebrew_aleph = 0xce0
XK_hebrew_bet = 0xce1
XK_hebrew_beth = 0xce1
XK_hebrew_gimel = 0xce2
XK_hebrew_gimmel = 0xce2
XK_hebrew_dalet = 0xce3
XK_hebrew_daleth = 0xce3
XK_hebrew_he = 0xce4
XK_hebrew_waw = 0xce5
XK_hebrew_zain = 0xce6
XK_hebrew_zayin = 0xce6
XK_hebrew_chet = 0xce7
XK_hebrew_het = 0xce7
XK_hebrew_tet = 0xce8
XK_hebrew_teth = 0xce8
XK_hebrew_yod = 0xce9
XK_hebrew_finalkaph = 0xcea
XK_hebrew_kaph = 0xceb
XK_hebrew_lamed = 0xcec
XK_hebrew_finalmem = 0xced
XK_hebrew_mem = 0xcee
XK_hebrew_finalnun = 0xcef
XK_hebrew_nun = 0xcf0
XK_hebrew_samech = 0xcf1
XK_hebrew_samekh = 0xcf1
XK_hebrew_ayin = 0xcf2
XK_hebrew_finalpe = 0xcf3
XK_hebrew_pe = 0xcf4
XK_hebrew_finalzade = 0xcf5
XK_hebrew_finalzadi = 0xcf5
XK_hebrew_zade = 0xcf6
XK_hebrew_zadi = 0xcf6
XK_hebrew_qoph = 0xcf7
XK_hebrew_kuf = 0xcf7
XK_hebrew_resh = 0xcf8
XK_hebrew_shin = 0xcf9
XK_hebrew_taw = 0xcfa
XK_hebrew_taf = 0xcfa
XK_Hebrew_switch = 0xFF7E
