# Nagstamon - Nagios status monitor for your desktop
# Copyright (C) 2008-2026 Henri Wahl <henri@nagstamon.de> et al.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

from Nagstamon.qui.qt import QIcon


class QIconWithFilename(QIcon):
    """
    Extends QIcon with a filename attribute.

    This class allows assigning the filename of the icon file as an attribute to a QIcon object.

    Args:
        *args: Arguments passed to QIcon. If the first argument is a string, it is stored as the filename.
        **kwargs: Keyword arguments passed to QIcon.

    Attributes:
        filename (str): The filename of the icon, if specified during creation.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if type(args[0]) == str:
            self.filename = args[0]
